/*  File: main.c,    Chen, Qihong    Apr 1, 1998
**
**  This program is used to perform disjoint sharp operation on two
**  arrays of cubes. Please note that this program only handles binary
**  variables.    
**
**  This program reads two arrays of cubes from two files. The file
**  format is similar to PLA file format, but it only support
**  ".i", ".e" and cubes in PLA format. There is only input cubes
**  in this file format. 
**
**  Example: the follow lines represent a array of cubes with 3 variables
**    .i 3
**    000
**    011
**    11-
**    .e
*/

#include <stdio.h>
#include <sys/time.h>
#include "cubeoper.h"

int debug = 0;

main(int argc, char *argv[])
\{ 
  ARRAYOFCUBES a, b, temp;
  FILE *fp1, *fp2;

  struct timeval  tv1,tv2,tvdiff;
  struct timezone tz;
  int res1, res2;
  
  /* initialize arrayofcubes */
  a.numvar  = 0;      a.numcube = 0;
  b.numvar  = 0;      b.numcube = 0;
  temp.numvar  = 0;   temp.numcube = 0;

  if (argc<3 || argc>4) 
   \{ printf("Usage: dsharp <infile1> <infile2> <d>\verb!\!n");
     exit(0);
   \}

  if (argc==4)
    if (strcmp(argv[3],"d")==0) debug=1;

  if ((fp1=fopen(argv[1],"r")) == 0)
   \{ printf("cannot open file %s\verb!\!n", argv[1]);
     exit(0);
   \}

  read_cubes(fp1, &a); 
  fclose(fp1);
  if (debug) showarray(&a, "A");

  if ((fp2=fopen(argv[2],"rw")) == 0)
   \{ printf("cannot open file %s\verb!\!n", argv[2]);
     exit(0);
   \}

  read_cubes(fp2, &b); 
  fclose(fp2);
  if (debug) showarray(&b, "B");

  res1 = gettimeofday(&tv1, &tz);
  dsharparr(&a, &b, &temp);
  res2 = gettimeofday(&tv2, &tz);
  showarray(&temp, "Result");

  if (debug)
   \{ if (! res1) 
       printf("start: %ld sec %ld usec\verb!\!n", tv1.tv_sec, tv1.tv_usec);
     else
       printf("first gettimeofday failed\verb!\!n");
   
     if (! res2)
       printf("  end: %ld sec %ld usec\verb!\!n", tv2.tv_sec, tv2.tv_usec);
     else
       printf("second gettimeofday failed\verb!\!n");
   \}

  difftv(&tv1, &tv2, &tvdiff);
  printf("time difference is %ld sec %ld usec\verb!\!n",tvdiff.tv_sec,tvdiff.tv_usec);
\}
